/*
Copyright 2015-2016, Manuel Bichler
WWW: <http://dbai.tuwien.ac.at/proj/lpopt/>

This file is part of lpopt.

lpopt is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

lpopt is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with lpopt.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef LOGICPROGRAMOPTIMIZATIONMANAGER_H_
#define LOGICPROGRAMOPTIMIZATIONMANAGER_H_

#include <string>
#include <vector>
#include <map>
#include <ctime>
#include <iostream>

#include <htd/main.hpp>

#include "../parser/Driver.h"

#include "../asp-domain/asp-domain.hpp"
#include "../asp-domain/StringGenerator.hpp"
#include "StatementOptimizationAlgorithm.hpp"
#include "predictor.hpp"

using namespace htd;

namespace lpopt
{
	enum DecompositionOrderingAlgorithm
	{
	        MCS, // MaximumCardinalitySearch
        	MF, // MinFill
	        MIW, // MinDegree
		NOA // NaturalOrdering
	};

	class LogicProgramOptimizationManager
	{
	public:
		LogicProgramOptimizationManager(std::istream *stream, DecompositionOrderingAlgorithm orderingAlg, bool ignoreHeadVariables);
		LogicProgramOptimizationManager(std::istream *stream, DecompositionOrderingAlgorithm orderingAlg, bool ignoreHeadVariables, std::istream *instStream, char* macroFileName, std::string keyListIn = "");
		~LogicProgramOptimizationManager();
	public:
		Program* optimize(); // optimizes the program from the stream. Caller has to delete returned program. If not already done, also parses the stream.
		unsigned int getMaxWidth() const; // returns size of biggest bag. Must only be called after calling optimize(), otherwise result is undefined.
		const Program& getBaseProgram(); // caller must not delete. If not already done, parses the stream.
	private:
		void checkParsedAndPreprocessed(); // does nothing, if already parsed and preprocessed. otherwise, calls parse() and preprocess().
		void parse(); // let driver parse and save into program
		void preprocess(); // inner domain optimization, pooling splitting of program
	private:
		string instanceString;
		const bool ignoreHeadVariables;
		parser::Driver parsingDriver;
		const DecompositionOrderingAlgorithm orderingAlg;
		Program *program; // if not NULL, the input stream has already been parsed.
//		clock_t htdlib_time, parse_time, me_time;
		unsigned int width;
		predictor Predictor;
		char* macroFile;
		std::string keyList;
	};
} // lpopt

#endif //LOGICPROGRAMOPTIMIZATIONMANAGER_H_


